/*	
**************************************************************************************
*	PS2Main.c to demonstrate the power of PS2Key.c and PS2Key.h drivers 
*	Date		: 17th June 2004
*	Programmer	: John Leung, www.TechToys.com.hk under Downloads Section.
*	Hardware    : Tested on Melab's X1 board dated 27-Jan 2006
*	Web page	: www.TechToys.com.hk
**************************************************************************************
*/

/*
**************************************************************************************
*	Date 3 Feb 2005
*	Bug reported when Xtal = 4MHz. When Xtal = 20MHz, the original code runs OK
*	Revission : 1. Change to 4MHz crystal
*				2. Add #use fast_io(b) in main()
*				3. Change A4 -> B1 for PS2Key_Data (for hardware simplicity only)
*				4. The key modification in void PS2KeyIntHandler(void). Previously
*				   switch (PS2Key_bit_Ix) case... applied but the assembly code
*				   generated took too long for clock signal detection of keyboard
*				   Instead of switch, a simple if() else if ... selection employed
*				5. Detection of PS2_BREAKCODE and PS2_SHIFT deleted from 
*				   PS2KeyIntHandler(void) to save interrupt time.
**************************************************************************************
*/

#include <16f877a.h>
#use delay(clock=4000000)
#use fast_io(b)
#fuses XT, NOWDT, NOLVP

#include <OS_CPU.h>				//typedef compatible with uCOS-II style

#include <PS2Key.h>
#include <PS2ScanCode.h>
#include <PS2Key.c>
#include <lcd_x1.c>

INT8U PS2Key_Char;

#int_EXT
void ext_isr(void)
{
	PS2KeyIntHandler();
}

void main(void)
{
	INT8U cnt;
	INT8U ptr = 1;

	port_b_pullups(TRUE);
	setup_adc_ports(NO_ANALOGS);

	PS2KeyInit();				//Init PS2 Keyboard
    lcd_init();					//Init 2x20 characters LCD module

	lcd_putc('\f');				//clear the module
	lcd_gotoxy(1,1);			//goto position 1,1 of the LCD module
	lcd_putc("PS2 Keypad Test");//Put a fixed string on LCD

	enable_interrupts(global);	//enable global interrupts

	for(;;)
	{
		if(PS2KeyHit())
		{
			PS2Key_Char = PS2KeyGetKey();
			if (PS2Key_Char==0xF0)							//Take care of BREAKCODE
				PS2_BREAKCODE = TRUE;
			else if (PS2_BREAKCODE==TRUE)
				PS2_BREAKCODE=FALSE; 						//Trashed BREAKCODE
			else if (PS2Key_Char!=0xE0)						//Handle special key like '/'		
				{ PS2Key_Char = PS2KeyNumPad[PS2Key_Char-OFFSET_SCH2];	//PS2KeyNumPad[] ONLY, & OFFSET_SCH2 from PS2ScanCode.h
		
				switch (PS2Key_Char){
					case 0x0D:
					//ENT Key, should print tag
					//function key, do the function you want
						break;
					case 0x77:
					//NumLock, another function key, like clearing the display
						for(cnt=1;cnt<21;cnt++) {lcd_gotoxy(cnt,2);lcd_putc(' ');}
							ptr=1; //reset the position pointer
						break;
					default:
					//Handle the numeric input, display to LCD for a num-pad
						lcd_gotoxy(ptr++,2);
						if(ptr>20) ptr=1;
					lcd_putc(PS2Key_Char);
				}	//end of switch 
			} 		//end of last else case of if(PS2Key_Char==0xF0)
		}			//end of if(PS2KeyHit())
	}				//for(;;)
}
